@@ -15,7 +15,7 @@
 # Third, the output is collected and combined.
 #
 # witten by (primarily) T Wolfers, (adaptated) SM Kia, H Huijsdens, L Parks, 
-# AF Marquand
+# S Rutherford, AF Marquand
 # -----------------------------------------------------------------------------
 
 from __future__ import print_function
@@ -27,22 +27,27 @@ import glob
 import shutil
 import pickle
 import fileinput
+import time
 import numpy as np
 import pandas as pd
-from subprocess import call
+from subprocess import call, check_output
+
 
 try:
     import pcntoolkit as ptk
-    import pcntoolkit.fileio as fileio
+    import pcntoolkit.dataio.fileio as fileio
     from pcntoolkit import configs
+    from pcntoolkit.util.utils import yes_or_no 
+    ptkpath = ptk.__path__[0] 
 except ImportError:
     pass
-    path = os.path.abspath(os.path.dirname(__file__))
-    if path not in sys.path:
-        sys.path.append(path)
-        del path
-    import fileio
+    ptkpath = os.path.abspath(os.path.dirname(__file__))
+    if ptkpath not in sys.path:
+        sys.path.append(ptkpath)
+    import dataio.fileio as fileio
     import configs
+    from util.utils import yes_or_no 
+    
     
 PICKLE_PROTOCOL = configs.PICKLE_PROTOCOL
 
@@ -57,50 +62,43 @@ def execute_nm(processing_dir,
                duration,
                normative_path=None,
                func='estimate',
+               interactive=False,
                **kwargs):
 
-    """
+    ''' Execute parallel normative models
     This function is a mother function that executes all parallel normative
     modelling routines. Different specifications are possible using the sub-
     functions.
 
-    :Parameters:
-        * processing_dir     -> Full path to the processing dir
-        * python_path        -> Full path to the python distribution
-        * normative_path     -> Full path to the normative.py. If None (default)
-                                then it will automatically retrieves the path from 
-                                the installed packeage.
-        * job_name           -> Name for the bash script that is the output of
-                                this function
-        * covfile_path       -> Full path to a .txt file that contains all
-                                covariats (subjects x covariates) for the
-                                responsefile
-        * respfile_path      -> Full path to a .txt that contains all features
-                                (subjects x features)
-        * batch_size         -> Number of features in each batch
-        * memory             -> Memory requirements written as string
-                                for example 4gb or 500mb
-        * duation            -> The approximate duration of the job, a string
-                                with HH:MM:SS for example 01:01:01
-        * cv_folds           -> Number of cross validations
-        * testcovfile_path   -> Full path to a .txt file that contains all
-                                covariats (subjects x covariates) for the
-                                testresponse file
-        * testrespfile_path  -> Full path to a .txt file that contains all
-                                test features
-        * log_path           -> Pathfor saving log files
-        * binary             -> If True uses binary format for response file
-                                otherwise it is text
+    Basic usage::
 
-    written by (primarily) T Wolfers, (adapted) SM Kia
-    """
+        execute_nm(processing_dir, python_path, job_name, covfile_path, respfile_path, batch_size, memory, duration)
+
+    :param processing_dir: Full path to the processing dir
+    :param python_path: Full path to the python distribution
+    :param normative_path: Full path to the normative.py. If None (default) then it will automatically retrieves the path from the installed packeage.
+    :param job_name: Name for the bash script that is the output of this function
+    :param covfile_path: Full path to a .txt file that contains all covariats (subjects x covariates) for the responsefile
+    :param respfile_path: Full path to a .txt that contains all features (subjects x features)
+    :param batch_size: Number of features in each batch
+    :param memory: Memory requirements written as string for example 4gb or 500mb
+    :param duation: The approximate duration of the job, a string with HH:MM:SS for example 01:01:01
+    :param cv_folds: Number of cross validations
+    :param testcovfile_path: Full path to a .txt file that contains all covariates (subjects x covariates) for the test response file
+    :param testrespfile_path: Full path to a .txt file that contains all test features
+    :param log_path: Path for saving log files
+    :param binary: If True uses binary format for response file otherwise it is text
+
+    written by (primarily) T Wolfers, (adapted) SM Kia, (adapted) S Rutherford.
+    '''
     
     if normative_path is None:
-        normative_path = ptk.__path__[0] + '/normative.py'
+        normative_path = ptkpath + '/normative.py'
         
     cv_folds = kwargs.get('cv_folds', None)
     testcovfile_path = kwargs.get('testcovfile_path', None)
     testrespfile_path= kwargs.get('testrespfile_path', None)
+    outputsuffix = kwargs.get('outputsuffix', 'estimate')
     cluster_spec = kwargs.pop('cluster_spec', 'torque')
     log_path = kwargs.pop('log_path', None)
     binary = kwargs.pop('binary', False)
@@ -122,8 +120,8 @@ def execute_nm(processing_dir,
         file_extentions = '.txt'
     
     kwargs.update({'batch_size':str(batch_size)})
+    job_ids = []
     for n in range(1, number_of_batches+1):
-        print(n)
         kwargs.update({'job_id':str(n)})
         if testrespfile_path is not None:
             if cv_folds is not None:
@@ -151,10 +149,11 @@ def execute_nm(processing_dir,
                                 batch_respfile_path,
                                 func=func,
                                 **kwargs)
-                    qsub_nm(job_path=batch_job_path,
+                    job_id = qsub_nm(job_path=batch_job_path,
                             log_path=log_path,
                             memory=memory,
                             duration=duration)
+                    job_ids.append(job_id)
                 elif cluster_spec == 'sbatch':
                     # update the response file 
                     kwargs.update({'testrespfile_path': \
@@ -194,10 +193,11 @@ def execute_nm(processing_dir,
                                 batch_respfile_path,
                                 func=func,
                                 **kwargs)
-                    qsub_nm(job_path=batch_job_path,
+                    job_id = qsub_nm(job_path=batch_job_path,
                             log_path=log_path,
                             memory=memory,
                             duration=duration)
+                    job_ids.append(job_id)
                 elif cluster_spec == 'sbatch':
                     sbatchwrap_nm(batch_processing_dir,
                                 python_path,
@@ -235,10 +235,11 @@ def execute_nm(processing_dir,
                                 batch_respfile_path,
                                 func=func,
                                 **kwargs)
-                    qsub_nm(job_path=batch_job_path,
+                    job_id = qsub_nm(job_path=batch_job_path,
                             log_path=log_path,
                             memory=memory,
                             duration=duration)
+                    job_ids.append(job_id)
                 elif cluster_spec == 'sbatch':
                     sbatchwrap_nm(batch_processing_dir,
                                 python_path,
@@ -259,32 +260,81 @@ def execute_nm(processing_dir,
                     qsub_nm(processing_dir=batch_processing_dir)
                     # ]
 
+    if interactive:
+        
+        check_jobs(job_ids, delay=60)
+        
+        success = False
+        while (not success):
+            success = collect_nm(processing_dir,
+                           job_name,
+                           func=func,
+                           collect=False,
+                           binary=binary,
+                           batch_size=batch_size,
+                           outputsuffix=outputsuffix)
+            if success:
+                break
+            else:
+                if interactive == 'query':
+                    response = yes_or_no('Rerun the failed jobs?')
+                    if response:
+                        rerun_nm(processing_dir, log_path=log_path, memory=memory, 
+                                 duration=duration, binary=binary, 
+                                 interactive=interactive)
+                    else:
+                        success = True
+                else:
+                    print('Reruning the failed jobs ...')
+                    rerun_nm(processing_dir, log_path=log_path, memory=memory, 
+                                 duration=duration, binary=binary, 
+                                 interactive=interactive)
+                    
+        if interactive == 'query':
+            response = yes_or_no('Collect the results?')
+            if response:
+                success = collect_nm(processing_dir,
+                               job_name,
+                               func=func,
+                               collect=True,
+                               binary=binary,
+                               batch_size=batch_size,
+                               outputsuffix=outputsuffix)
+        else:
+            print('Collecting the results ...')
+            success = collect_nm(processing_dir,
+                               job_name,
+                               func=func,
+                               collect=True,
+                               binary=binary,
+                               batch_size=batch_size,
+                               outputsuffix=outputsuffix)
+
 
 """routines that are environment independent"""
 
-
 def split_nm(processing_dir,
              respfile_path,
              batch_size,
              binary,
              **kwargs):
 
-    """ This function prepares the input files for normative_parallel.
+    ''' This function prepares the input files for normative_parallel.
+    
+    Basic usage::
 
-    :Parameters:
-        * processing_dir    -> Full path to the folder of processing
-        * respfile_path     -> Full path to the responsefile.txt
-                               (subjects x features)
-        * batch_size        -> Number of features in each batch
-        * testrespfile_path -> Full path to the test responsefile.txt
-                               (subjects x features)
-        * binary            -> If True binary file
+        split_nm(processing_dir, respfile_path, batch_size, binary, testrespfile_path)
 
-    :outputs:
-        * The creation of a folder struture for batch-wise processing
+    :param processing_dir: Full path to the processing dir
+    :param respfile_path: Full path to the responsefile.txt (subjects x features)
+    :param batch_size: Number of features in each batch
+    :param testrespfile_path: Full path to the test responsefile.txt (subjects x features)
+    :param binary: If True binary file
 
-    witten by (primarily) T Wolfers (adapted) SM Kia
-    """
+    :outputs: The creation of a folder struture for batch-wise processing.
+
+    witten by (primarily) T Wolfers (adapted) SM Kia, (adapted) S Rutherford.
+    ''' 
     
     testrespfile_path = kwargs.pop('testrespfile_path', None)
 
@@ -387,21 +437,24 @@ def collect_nm(processing_dir,
                batch_size=None,
                outputsuffix='_estimate'):
     
-    """This function checks and collects all batches.
+    '''Function to checks and collects all batches.
 
-    :Parameters:
-        * processing_dir        -> Full path to the processing directory
-        * collect               -> If True data is checked for failed batches
-                                and collected; if False data is just checked
-        * binary                -> Results in pkl format? 
+    Basic usage::
 
-    :ouptuts:
-        * Text files containing all results accross all batches the combined
-          output (written to disk)
-        * returns 0 if batches fail, 1 otherwise
+        collect_nm(processing_dir, job_name)
 
-    written by (primarily) T Wolfers, (adapted) SM Kia
-    """
+
+    :param processing_dir: Full path to the processing directory
+    :param collect: If True data is checked for failed batches and collected; if False data is just checked
+    :param binary: Results in pkl format
+
+    :outputs: Text files containing all results accross all batches the combined output (written to disk).
+
+    :returns 0: if batches fail
+    :returns 1: if bathches complete successfully
+
+    written by (primarily) T Wolfers, (adapted) SM Kia, (adapted) S Rutherford.
+    '''
 
     if binary:
         file_extentions = '.pkl'
@@ -414,7 +467,7 @@ def collect_nm(processing_dir,
     count = 0
     batch_fail = []
     
-    if func != 'fit':
+    if (func!='fit' and func!='extend' and func!='merge' and func!='tune'):
         file_example = []
         # TODO: Collect_nm only depends on yhat, thus does not work when no 
         # prediction is made (when test cov is not specified). 
@@ -492,6 +545,18 @@ def collect_nm(processing_dir,
                     Z = pd.DataFrame(Z)
                     fileio.save(Z, batch + 'Z' + outputsuffix + 
                                 file_extentions)
+                    
+                    nll = np.zeros(batch_size)
+                    nll = nll.transpose()
+                    nll = pd.Series(nll)
+                    fileio.save(nll, batch + 'NLL' + outputsuffix + 
+                                file_extentions)
+                    
+                    bic = np.zeros(batch_size)
+                    bic = bic.transpose()
+                    bic = pd.Series(bic)
+                    fileio.save(bic, batch + 'BIC' + outputsuffix + 
+                                file_extentions)
     
                     if not os.path.isdir(batch + 'Models'):
                         os.mkdir('Models')
@@ -626,8 +691,32 @@ def collect_nm(processing_dir,
             fileio.save(msll_dfs, processing_dir + 'MSLL' + outputsuffix +
                         file_extentions)
             del msll_dfs
+            
+        nll_filenames = glob.glob(processing_dir + 'batch_*/' + 'NLL' +
+                                  outputsuffix + '*')
+        if nll_filenames:
+            nll_filenames = fileio.sort_nicely(nll_filenames)
+            nll_dfs = []
+            for nll_filename in nll_filenames:
+                nll_dfs.append(pd.DataFrame(fileio.load(nll_filename)))
+            nll_dfs = pd.concat(nll_dfs, ignore_index=True, axis=0)
+            fileio.save(nll_dfs, processing_dir + 'NLL' + outputsuffix +
+                        file_extentions)
+            del nll_dfs
+
+        bic_filenames = glob.glob(processing_dir + 'batch_*/' + 'BIC' +
+                                  outputsuffix + '*')
+        if bic_filenames:
+            bic_filenames = fileio.sort_nicely(bic_filenames)
+            bic_dfs = []
+            for bic_filename in bic_filenames:
+                bic_dfs.append(pd.DataFrame(fileio.load(bic_filename)))
+            bic_dfs = pd.concat(bic_dfs, ignore_index=True, axis=0)
+            fileio.save(bic_dfs, processing_dir + 'BIC' + outputsuffix +
+                        file_extentions)
+            del bic_dfs
         
-        if func != 'predict' and func != 'extend':
+        if (func!='predict' and func!='extend' and func!='merge' and func!='tune'):
             if not os.path.isdir(processing_dir + 'Models') and \
                os.path.exists(os.path.join(batches[0], 'Models')):
                 os.mkdir(processing_dir + 'Models')
@@ -654,8 +743,8 @@ def collect_nm(processing_dir,
                     if meta_data['outscaler'] in ['standardize', 'minmax', 
                                 'robminmax']:
                         Y_scalers.append(meta_data['scaler_resp'])
-                meta_data['mean_resp'] = np.squeeze(np.stack(mY)) 
-                meta_data['std_resp'] = np.squeeze(np.stack(sY))
+                meta_data['mean_resp'] = np.squeeze(np.column_stack(mY)) 
+                meta_data['std_resp'] = np.squeeze(np.column_stack(sY))
                 meta_data['scaler_cov'] = X_scalers 
                 meta_data['scaler_resp'] = Y_scalers
                 
@@ -697,21 +786,23 @@ def collect_nm(processing_dir,
             file_extentions)
 
     if not batch_fail:
-        return 1
+        return True
     else:
-        return 0
+        return False
 
 def delete_nm(processing_dir,
               binary=False):
-    """This function deletes all processing for normative modelling and just
-    keeps the combined output.
+    '''This function deletes all processing for normative modelling and just keeps the combined output.
 
-    :Parameters:
-        * processing_dir        -> Full path to the processing directory
-        * binary                -> Results in pkl format? 
+    Basic usage::
 
-    written by (primarily) T Wolfers, (adapted) SM Kia
-    """
+        collect_nm(processing_dir)
+
+    :param processing_dir: Full path to the processing directory.
+    :param binary: Results in pkl format.
+
+    written by (primarily) T Wolfers, (adapted) SM Kia, (adapted) S Rutherford.
+    '''
     
     if binary:
         file_extentions = '.pkl'
@@ -736,35 +827,29 @@ def bashwrap_nm(processing_dir,
                 func='estimate',
                 **kwargs):
 
-    """ This function wraps normative modelling into a bash script to run it
+    ''' This function wraps normative modelling into a bash script to run it
     on a torque cluster system.
 
-    :Parameters:
-        * processing_dir     -> Full path to the processing dir
-        * python_path        -> Full path to the python distribution
-        * normative_path     -> Full path to the normative.py
-        * job_name           -> Name for the bash script that is the output of
-                                this function
-        * covfile_path       -> Full path to a .txt file that contains all
-                                covariats (subjects x covariates) for the
-                                responsefile
-        * respfile_path      -> Full path to a .txt that contains all features
-                                (subjects x features)
-        * cv_folds           -> Number of cross validations
-        * testcovfile_path   -> Full path to a .txt file that contains all
-                                covariats (subjects x covariates) for the
-                                testresponse file
-        * testrespfile_path  -> Full path to a .txt file that contains all
-                                test features
-        * alg                -> which algorithm to use
-        * configparam        -> configuration parameters for this algorithm
+    Basic usage::
 
-    :outputs:
-        * A bash.sh file containing the commands for normative modelling saved
-          to the processing directory (written to disk)
+        bashwrap_nm(processing_dir, python_path, normative_path, job_name, covfile_path, respfile_path)
 
-    written by (primarily) T Wolfers
-    """
+    :param processing_dir: Full path to the processing dir
+    :param python_path: Full path to the python distribution
+    :param normative_path: Full path to the normative.py
+    :param job_name: Name for the bash script that is the output of this function
+    :param covfile_path: Full path to a .txt file that contains all covariates (subjects x covariates) for the responsefile
+    :param respfile_path: Full path to a .txt that contains all features (subjects x features)
+    :param cv_folds: Number of cross validations
+    :param testcovfile_path: Full path to a .txt file that contains all covariates (subjects x covariates) for the testresponse file
+    :param testrespfile_path: Full path to a .txt file that contains all test features
+    :param alg: which algorithm to use
+    :param configparam: configuration parameters for this algorithm
+
+    :outputs: A bash.sh file containing the commands for normative modelling saved to the processing directory (written to disk).
+
+    written by (primarily) T Wolfers, (adapted) S Rutherford.
+    '''
     
     # here we use pop not get to remove the arguments as they used 
     cv_folds = kwargs.pop('cv_folds',None)
@@ -772,7 +857,6 @@ def bashwrap_nm(processing_dir,
     testrespfile_path = kwargs.pop('testrespfile_path', None)
     alg = kwargs.pop('alg', None)
     configparam = kwargs.pop('configparam', None)
-    standardize = kwargs.pop('standardize', True)
     
     # change to processing dir
     os.chdir(processing_dir)
@@ -807,8 +891,8 @@ def bashwrap_nm(processing_dir,
             job_call = [job_call[0] + ' -x ' + str(configparam)]
     
     # add standardization flag if it is false
-    if not standardize:
-        job_call = [job_call[0] + ' -s']
+    # if not standardize:
+    #     job_call = [job_call[0] + ' -s']
     
     # add responses file
     job_call = [job_call[0] + ' ' + respfile_path]
@@ -830,23 +914,23 @@ def qsub_nm(job_path,
             log_path,
             memory,
             duration):
-    """
-    This function submits a job.sh scipt to the torque custer using the qsub
-    command.
+    
+    '''This function submits a job.sh scipt to the torque custer using the qsub command.
+    
+    Basic usage::
 
-    ** Input:
-        * job_path      -> Full path to the job.sh file
-        * memory        -> Memory requirements written as string for example
-                           4gb or 500mb
-        * duation       -> The approximate duration of the job, a string with
-                           HH:MM:SS for example 01:01:01
 
-    ** Output:
-        * Submission of the job to the (torque) cluster
+        qsub_nm(job_path, log_path, memory, duration)
 
-    witten by (primarily) T Wolfers, (adapted) SM Kia
-    """
+    :param job_path: Full path to the job.sh file.
+    :param memory: Memory requirements written as string for example 4gb or 500mb.
+    :param duation: The approximate duration of the job, a string with HH:MM:SS for example 01:01:01.
 
+    :outputs: Submission of the job to the (torque) cluster.
+
+    written by (primarily) T Wolfers, (adapted) SM Kia, (adapted) S Rutherford.
+    '''
+  
     # created qsub command
     if log_path is None:
         qsub_call = ['echo ' + job_path + ' | qsub -N ' + job_path + ' -l ' +
@@ -857,27 +941,32 @@ def qsub_nm(job_path,
                      duration + ' -o ' + log_path + ' -e ' + log_path]
 
     # submits job to cluster
-    call(qsub_call, shell=True)
+    #call(qsub_call, shell=True)
+    job_id = check_output(qsub_call, shell=True).decode(sys.stdout.encoding).replace("\n", "")
+    
+    return job_id
 
 
 def rerun_nm(processing_dir,
              log_path,
              memory,
              duration,
-             binary=False):
-    """
-    This function reruns all failed batched in processing_dir after collect_nm
-    has identified he failed batches
+             binary=False, 
+             interactive=False):
+    '''This function reruns all failed batched in processing_dir after collect_nm has identified the failed batches.
+    Basic usage::           
 
-    * Input:
-        * processing_dir        -> Full path to the processing directory
-        * memory                -> Memory requirements written as string
-                                   for example 4gb or 500mb
-        * duration               -> The approximate duration of the job, a
-                                   string with HH:MM:SS for example 01:01:01
+        rerun_nm(processing_dir, log_path, memory, duration)
 
-    written by (primarily) T Wolfers, (adapted) SM Kia
-    """
+    :param processing_dir: Full path to the processing directory
+    :param memory: Memory requirements written as string for example 4gb or 500mb.
+    :param duration: The approximate duration of the job, a string with HH:MM:SS for example 01:01:01.
+
+    written by (primarily) T Wolfers, (adapted) SM Kia, (adapted) S Rutherford.
+    '''
+    
+    job_ids = []
+    
 
     if binary:
         file_extentions = '.pkl'
@@ -887,10 +976,11 @@ def rerun_nm(processing_dir,
         for n in range(0, shape[0]):
             jobpath = failed_batches[n, 0]
             print(jobpath)
-            qsub_nm(job_path=jobpath,
+            job_id = qsub_nm(job_path=jobpath,
                     log_path=log_path,
                     memory=memory,
                     duration=duration)
+            job_ids.append(job_id)
     else:
         file_extentions = '.txt'
         failed_batches = fileio.load_pd(processing_dir +
@@ -899,10 +989,15 @@ def rerun_nm(processing_dir,
         for n in range(0, shape[0]):
             jobpath = failed_batches.iloc[n, 0]
             print(jobpath)
-            qsub_nm(job_path=jobpath,
+            job_id = qsub_nm(job_path=jobpath,
                     log_path=log_path,
                     memory=memory,
                     duration=duration)
+            job_ids.append(job_id)
+            
+    if interactive: 
+        check_jobs(job_ids, delay=60)
+        
 
 # COPY the rotines above here and aadapt those to your cluster
 # bashwarp_nm; qsub_nm; rerun_nm
@@ -918,35 +1013,29 @@ def sbatchwrap_nm(processing_dir,
                   func='estimate',
                   **kwargs):
 
-    """ This function wraps normative modelling into a bash script to run it
+    '''This function wraps normative modelling into a bash script to run it
     on a torque cluster system.
 
-    :Parameters:
-        * processing_dir     -> Full path to the processing dir
-        * python_path        -> Full path to the python distribution
-        * normative_path     -> Full path to the normative.py
-        * job_name           -> Name for the bash script that is the output of
-                                this function
-        * covfile_path       -> Full path to a .txt file that contains all
-                                covariats (subjects x covariates) for the
-                                responsefile
-        * respfile_path      -> Full path to a .txt that contains all features
-                                (subjects x features)
-        * cv_folds           -> Number of cross validations
-        * testcovfile_path   -> Full path to a .txt file that contains all
-                                covariats (subjects x covariates) for the
-                                testresponse file
-        * testrespfile_path  -> Full path to a .txt file that contains all
-                                test features
-        * alg                -> which algorithm to use
-        * configparam        -> configuration parameters for this algorithm
+    Basic usage::
 
-    :outputs:
-        * A bash.sh file containing the commands for normative modelling saved
-          to the processing directory (written to disk)
+        sbatchwrap_nm(processing_dir, python_path, normative_path, job_name, covfile_path, respfile_path, memory, duration)
 
-    written by (primarily) T Wolfers
-    """
+    :param processing_dir: Full path to the processing dir
+    :param python_path: Full path to the python distribution
+    :param normative_path: Full path to the normative.py
+    :param job_name: Name for the bash script that is the output of this function
+    :param covfile_path: Full path to a .txt file that contains all covariates (subjects x covariates) for the responsefile
+    :param respfile_path: Full path to a .txt that contains all features (subjects x features)
+    :param cv_folds: Number of cross validations
+    :param testcovfile_path: Full path to a .txt file that contains all covariates (subjects x covariates) for the testresponse file
+    :param testrespfile_path: Full path to a .txt file that contains all test features
+    :param alg: which algorithm to use
+    :param configparam: configuration parameters for this algorithm
+
+    :outputs: A bash.sh file containing the commands for normative modelling saved to the processing directory (written to disk).
+
+    written by (primarily) T Wolfers, (adapted) S Rutherford
+    '''
     
     # here we use pop not get to remove the arguments as they used 
     cv_folds = kwargs.pop('cv_folds',None)
@@ -954,7 +1043,6 @@ def sbatchwrap_nm(processing_dir,
     testrespfile_path = kwargs.pop('testrespfile_path', None)
     alg = kwargs.pop('alg', None)
     configparam = kwargs.pop('configparam', None)
-    standardize = kwargs.pop('standardize', True)
     
     # change to processing dir
     os.chdir(processing_dir)
@@ -1008,8 +1096,8 @@ def sbatchwrap_nm(processing_dir,
             job_call = [job_call[0] + ' -x ' + str(configparam)]
     
     # add standardization flag if it is false
-    if not standardize:
-        job_call = [job_call[0] + ' -s']
+    # if not standardize:
+    #     job_call = [job_call[0] + ' -s']
     
     # add responses file
     job_call = [job_call[0] + ' ' + respfile_path]
@@ -1028,19 +1116,21 @@ def sbatchwrap_nm(processing_dir,
 
 def sbatch_nm(job_path,
               log_path):
-    """
-    This function submits a job.sh scipt to the torque custer using the qsub
+    
+    '''This function submits a job.sh scipt to the torque custer using the qsub
     command.
 
-    ** Input:
-        * job_path      -> Full path to the job.sh file
-        * log_path      -> The logs are currently stored in the working dir
+    Basic usage::
 
-    ** Output:
-        * Submission of the job to the (torque) cluster
+        sbatch_nm(job_path, log_path)
 
-    witten by (primarily) T Wolfers
-    """
+    :param job_path: Full path to the job.sh file
+    :param log_path: The logs are currently stored in the working dir
+
+    :outputs: Submission of the job to the (torque) cluster.
+
+    written by (primarily) T Wolfers, (adapted) S Rutherford.
+    '''
 
     # created qsub command
     sbatch_call = ['sbatch ' + job_path]
@@ -1048,68 +1138,140 @@ def sbatch_nm(job_path,
     # submits job to cluster
     call(sbatch_call, shell=True)
     
-    def rerun_nm(processing_dir,
+def rerun_nm(processing_dir,
                  memory,
                  duration,
                  new_memory=False,
                  new_duration=False,
                  binary=False,
                  **kwargs):
-        """
-        This function reruns all failed batched in processing_dir after 
-        collect_nm has identified he failed batches
+        
+    '''This function reruns all failed batched in processing_dir after collect_nm has identified he failed batches.
     
-        * Input:
-            * processing_dir        -> Full path to the processing directory
-            * memory                -> Memory requirements written as string
-                                       for example 4gb or 500mb
-            * duration              -> The approximate duration of the job, a
-                                       string with HH:MM:SS for example 01:01:01
-            * new_memory            -> If you want to change the memory 
-                                        you have to indicate it here.
-            * new_duration          -> If you want to change the duration 
-                                        you have to indicate it here.
-        * Outputs:
-            * Reruns failed batches. 
+    Basic usage::
+
+        rerun_nm(processing_dir, memory, duration)
+
+    :param processing_dir: Full path to the processing directory.
+    :param memory: Memory requirements written as string, for example 4gb or 500mb.
+    :param duration: The approximate duration of the job, a string with HH:MM:SS for example 01:01:01.
+    :param new_memory: If you want to change the memory you have to indicate it here.
+    :param new_duration: If you want to change the duration you have to indicate it here.
+
+    :outputs: Re-runs failed batches. 
     
-        written by (primarily) T Wolfers
-        """
-        log_path = kwargs.pop('log_path', None)
+     written by (primarily) T Wolfers, (adapted) S Rutherford.
+    '''
+    log_path = kwargs.pop('log_path', None)
     
-        if binary:
-            file_extentions = '.pkl'
-            failed_batches = fileio.load(processing_dir +
-                                         'failed_batches' + 
-                                         file_extentions)
-            shape = failed_batches.shape
-            for n in range(0, shape[0]):
-                jobpath = failed_batches[n, 0]
-                print(jobpath)
-                if new_duration != False:
+    if binary:
+        file_extentions = '.pkl'
+        failed_batches = fileio.load(processing_dir + 'failed_batches' +  file_extentions)
+        shape = failed_batches.shape
+        for n in range(0, shape[0]):
+            jobpath = failed_batches[n, 0]
+            print(jobpath)
+            if new_duration != False:
+                with fileinput.FileInput(jobpath, inplace=True) as file:
+                    for line in file:
+                        print(line.replace(duration, new_duration), end='')
+                if new_memory != False:
                     with fileinput.FileInput(jobpath, inplace=True) as file:
                         for line in file:
-                            print(line.replace(duration, new_duration), end='')
-                if new_memory != False:
-                    with fileinput.FileInput(jobpath, inplace=True) as file:
-                        for line in file:
                             print(line.replace(memory, new_memory), end='')
-                sbatch_nm(jobpath,
-                          log_path)
-        else:
-            file_extentions = '.txt'
-            failed_batches = fileio.load_pd(processing_dir +
-                                           'failed_batches' + file_extentions)
-            shape = failed_batches.shape
-            for n in range(0, shape[0]):
-                jobpath = failed_batches.iloc[n, 0]
-                print(jobpath)
-                if new_duration != False:
-                    with fileinput.FileInput(jobpath, inplace=True) as file:
-                        for line in file:
-                            print(line.replace(duration, new_duration), end='')
+                sbatch_nm(jobpath, log_path)
+
+    else:
+        file_extentions = '.txt'
+        failed_batches = fileio.load_pd(processing_dir + 'failed_batches' + file_extentions)
+        shape = failed_batches.shape
+        for n in range(0, shape[0]):
+            jobpath = failed_batches.iloc[n, 0]
+            print(jobpath)
+            if new_duration != False:
+                with fileinput.FileInput(jobpath, inplace=True) as file:
+                    for line in file:
+                        print(line.replace(duration, new_duration), end='')
                 if new_memory != False:
                     with fileinput.FileInput(jobpath, inplace=True) as file:
                         for line in file:
                             print(line.replace(memory, new_memory), end='')
                 sbatch_nm(jobpath,
                           log_path)
+
+
+def retrieve_jobs():
+    """
+    A utility function to retrieve task status from the outputs of qstat.
+    
+    :return: a dictionary of jobs.
+
+    """
+    
+    output = check_output('qstat', shell=True).decode(sys.stdout.encoding)
+    output = output.split('\n')
+    jobs = dict()
+    for line in output[2:-1]:
+        (Job_ID, Job_Name, User, Wall_Time, Status, Queue) = line.split()
+        jobs[Job_ID] = dict()
+        jobs[Job_ID]['name'] = Job_Name
+        jobs[Job_ID]['walltime'] = Wall_Time
+        jobs[Job_ID]['status'] = Status
+        
+    return jobs
+        
+
+def check_job_status(jobs):
+    """
+    A utility function to count the tasks with different status.
+    
+    :param jobs: List of job ids.
+    :return: returns the number of taks athat are queued, running, completed,
+    and other status.
+    
+    """
+    running_jobs = retrieve_jobs()
+    
+    r = 0
+    c = 0
+    q = 0
+    u = 0
+    for job in jobs:
+        try:
+            if running_jobs[job]['status'] == 'C':
+                c += 1
+            elif running_jobs[job]['status'] == 'Q':
+                q += 1
+            elif running_jobs[job]['status'] == 'R':
+                r += 1
+            else:
+                u += 1
+        except: # probably meanwhile the job is finished.
+            c += 1 
+            continue
+                 
+    print('Total Jobs:%d, Queued:%d, Running:%d, Completed:%d, Unknown:%d' 
+          %(len(jobs), q, r, c, u))
+    return q,r,c,u
+    
+
+def check_jobs(jobs, delay=60):
+    """
+    A utility function for chacking the status of submitted jobs.
+    
+    :param jobs: list of job ids.
+    :param delay: the delay (in seconds) between two consequative checks, 
+    defaults to 60.
+
+    """
+    
+    n = len(jobs)
+    
+    while(True):
+        q,r,c,u = check_job_status(jobs)
+        if c == n:
+            print('All jobs are completed!')
+            break
+        time.sleep(delay)
+        
+
